CodeAssetExtension class final

The protocol extension for the hook/build.dart and hook/link.dart with CodeAssets and CodeConfig.

Constructors

CodeAssetExtension({required Architecture targetArchitecture, required OS targetOS, required LinkModePreference linkModePreference, CCompilerConfig? cCompiler, AndroidCodeConfig? android, IOSCodeConfig? iOS, MacOSCodeConfig? macOS, Sanitizer? sanitizer})
Constructs a CodeAssetExtension.

Properties

android AndroidCodeConfig?
Optional Android specific configuration.
final
cCompiler CCompilerConfig?
Optional C compiler configuration.
final
hashCode int
The hash code for this object.
no setterinherited
iOS IOSCodeConfig?
Optional iOS specific configuration.
final
linkModePreference LinkModePreference
The preferred link mode.
final
logger → Logger
The logger of the hooks_runner that drives this extension.
no setterinherited
macOS MacOSCodeConfig?
Optional macOS specific configuration.
final
runtimeType Type
A representation of the runtime type of the object.
no setterinherited
sanitizer Sanitizer?
Optional compiler sanitizer to use when compiling code assets.
final
targetArchitecture Architecture
The architecture to build for.
final
targetOS OS
The operating system to build for.
final

Methods

noSuchMethod(Invocation invocation) → dynamic
Invoked when a nonexistent method or property is accessed.
inherited
outputFiles(List<EncodedAsset> assets) Iterable<Uri>
Returns any files emitted by the hook output for this extension.
setupBuildInput(BuildInputBuilder input) → void
Setup the BuildConfig for this extension.
setupLinkInput(LinkInputBuilder input) → void
Setup the HookConfig for this extension.
setupLogger(Logger logger) → void
Sets up the logger used by the hooks_runner that drives this extension.
inherited
toString() String
A string representation of this object.
inherited
validateApplicationAssets(List<EncodedAsset> assets) Future<ValidationErrors>
Reports errors on the complete set of assets after all hooks are run.
validateBuildInput(BuildInput input) Future<ValidationErrors>
Reports errors from this extension on the BuildInput.
validateBuildOutput(BuildInput input, BuildOutput output) Future<ValidationErrors>
Reports errors from this extension on the LinkInput.
validateLinkInput(LinkInput input) Future<ValidationErrors>
Reports errors from this extension on the LinkInput.
validateLinkOutput(LinkInput input, LinkOutput output) Future<ValidationErrors>
Reports errors from this extension on the LinkOutput.

Operators

operator ==(Object other) bool
The equality operator.
inherited